@@ -37,22 +37,26 @@
       (if pflag 
 	  (cons :primitive (cons (list 'POP 1 0) (cdr body)))
 	body)
-    (free-body (conv-body variables body pflag))))
+    (free-body (conv-body variables (strip-exp body *definitions*) pflag))))
 
-(defun phase1-trans-body (vars body pflag)      
-  (cond ((and (listp body) (eql (car body) 'nesl::base-typecase))
-	 `(,(first body) ,(second body)
-	   ,@(mapcar #'(lambda (a) 
-			 (list (first a) 
-			       (trans-phase1 vars (second a) pflag)))
-		     (cddr body))))
-	((and (listp body) (eql (car body) 'nesl::poly-typecase))
-	 `(,(first body) ,(second body)
-	   ,(trans-phase1 vars (third body) pflag)
-	   ,(trans-phase1 vars (fourth body) pflag)
-	   ,pflag
-	   ,@(cddddr body)))
-	(t (trans-phase1 vars body pflag))))
+(defun phase1-trans-body (names body type pflag)      
+  (let ((variables (cdr names)))
+    (cond ((and (listp body) (eql (car body) 'nesl::base-typecase))
+	   `(,(first body) ,(second body)
+	     ,@(mapcar #'(lambda (a) 
+			   (list (first a) 
+				 (trans-phase1 variables (second a) pflag)))
+		       (cddr body))))
+	  ((and (listp body) (eql (car body) 'nesl::poly-typecase))
+	   `(,(first body) ,(second body)
+	     ,(trans-phase1 variables (third body) pflag)
+	     ,(trans-phase1 variables (fourth body) pflag)
+	     ,(trans-phase1 variables (or (fifth body)
+					  (generate-default-pair-code 
+					   names
+					   type (second body)))
+			    pflag)))
+	  (t (trans-phase1 variables body pflag)))))
 
 (defun special-body? (body)
   (and (listp body)
@@ -62,20 +66,8 @@
 
 (defun parse-defop (arguments definitions)
   (let* ((names (first arguments))
-	 (args (if (eql (second arguments) 'nesl::!)
-		   (cddr arguments)
-		 (cons nil (cdr arguments))))
+	 (args (cdr arguments))
 	 (keys (cddr args)))
-    (declare (special *redefine-default*))
-    (when (or (not (listp names))
-	      (not (plusp (length names)))
-	      (not (symbolp (first names)))
-	      (< (length args) 2)
-	      (not (zerop (mod (length keys) 2))))
-      (nesl-error "Bad function definition: (DEFOP ~a ...).~%~
-                   The syntax for DEFOP is:~%~%  ~
-         (DEFOP (Ident Ident*) [! typespec]~%    exp)~%"
-		  (car arguments)))
     (make-defop names (first args) (second args)
 		(get-keyword :type-check keys t)
 		(get-keyword :documentation keys nil)
@@ -88,6 +80,7 @@
 (defun add-function (names type body pflag definitions
 			   &key documentation shortdoc example redefine infix)
   (let* ((ntype (cons (conv-type (car type) pflag) (cdr type)))
+	 (stype (strip-type ntype definitions))
 	 (nnames (conv-names names pflag))
 	 (olddef (get-fundef (car nnames) definitions)))
     (declare (special *redefine-default*))
@@ -98,17 +91,18 @@
 		       :documentation documentation
 		       :shortdoc shortdoc
 		       :type ntype
+		       :codetype stype
 		       :example example
 		       :redefine redefine
 		       :infix infix)
-    (finalize-function-def (phase1-trans-body (cdr names) body pflag)
+    (finalize-function-def (phase1-trans-body names body (car type) pflag)
 			   definitions)))
 
 (defun make-defop (names type body 
 			 type-check documentation example redefine shortdoc
 			 infix definitions)
   (let* ((*current-fundef* (car names))
-	 (simplified-type (if type (simplify-type type) nil))
+	 (simplified-type type)
 	 (check-valid-user-type 
 	  (check-type-list simplified-type definitions))
 	 (type-check (and type-check (not (special-body? body))))
@@ -136,31 +130,23 @@
     (cons (car list) (get-defrec-typebindings (cdr list)))))
 
 (defun parse-defrec (arguments definitions)
-  (let ((names (first arguments)))	       
-    (when (or (not (listp names))
-	      (not (plusp (length names)))
-	      (not (symbolp (first names)))
-	      (< (length arguments) 1))
-      (nesl-error "Bad type definition: (DEFREC ~a ...).~%~
-                   The syntax for DEFREC is:  ~
-         (DEFREC (Ident typeexp*) typebind*)."
-		  (car arguments)))
-    (let* ((typebindings (get-defrec-typebindings (cdr arguments)))
-	   (keys (nthcdr (length typebindings) (cdr arguments))))
-      (declare (special *redefine-default*))
-      (make-defrec names typebindings
-		   (get-keyword :documentation keys nil)
-		   (get-keyword :redefine keys *redefine-default*)
-		   definitions))))
+  (let* ((names (first arguments))
+	 (typebindings (get-defrec-typebindings (cdr arguments)))
+	 (keys (nthcdr (length typebindings) (cdr arguments))))
+    (declare (special *redefine-default*))
+    (make-defrec names typebindings
+		 (get-keyword :documentation keys nil)
+		 (get-keyword :redefine keys *redefine-default*)
+		 definitions)))
 
 (defun add-type (names type pflag definitions
 		       &key documentation redefine)
-  (add-function names type '(:primitive) pflag definitions
-		:documentation documentation
-		:redefine redefine)
   (add-type-def (car (conv-names names pflag) )
 		(cons (conv-type (car type) pflag) (cdr type))
-		definitions))
+		definitions)
+  (add-function names type '(:primitive) pflag definitions
+		:documentation documentation
+		:redefine redefine))
 
 (defun make-args (count)
   (do ((i count (- i 1))
@@ -181,8 +167,62 @@
 	 (doc (or documentation
 		  (format nil "Constructor for the record type ~a." name))))
     (declare (special *current-typedef*) (ignore check-valid-type))
-    (add-type full-names type t definitions)
     (add-type full-names type nil definitions
 	      :documentation doc
 	      :redefine redefine)
+    (add-type full-names type t definitions)
     (values full-names (cons 'function type))))
+
+
+#| *********************
+The following generates code for polymorphic functions on pairs.
+|# 
+
+(defun generate-default-pair-code (names funtype typevar)
+  ;;(print (list 'here names funtype typevar))
+  (multiple-value-bind (binds args1 args2)
+    (variable-bind-calls (second names) (second funtype) typevar)
+    (let ((calls (list (list (car names) args1)
+		       (list (car names) args2))))
+      `(with ,binds ,(combine-code calls (car funtype) typevar)))))
+
+(defun variable-bind-calls (args argtypes typevar)
+  (if (listp args)
+      (multiple-value-bind (bindsleft vleft1 vleft2)
+	  (variable-bind-calls (second args) (second argtypes) typevar)
+	(multiple-value-bind (bindsright vright1 vright2)
+	  (variable-bind-calls (third args) (third argtypes) typevar)
+	  (values (append bindsleft bindsright)
+		  `(nesl::pair ,vleft1 ,vright1)
+		  `(nesl::pair ,vleft2 ,vright2))))
+    (variable-bind-call args argtypes typevar)))
+
+(defun variable-bind-call (arg argtype typevar)
+  (cond ((eql argtype typevar)
+	 (let ((v1 (gensym)) 
+	       (v2 (gensym)))
+	   (values `(((nesl::pair ,v1 ,v2) ,arg))
+		   v1 v2)))
+	((and (listp argtype) (eql (car argtype) 'nesl::vector)
+	      (eql (second argtype) typevar))
+	 (let ((v1 (gensym))
+	       (v2 (gensym))
+	       (seg (gensym)))
+	   (values `(((nesl::vector 
+		    (nesl::pair ,seg (nesl::pair ,v1 ,v2))) ,arg))
+		 `(nesl::vector (nesl::pair ,seg ,v1))
+		 `(nesl::vector (nesl::pair ,seg ,v2)))))
+	(t 
+	 (values nil arg arg))))
+
+(defun combine-code (calls return-type typevar)
+  (cond ((eql return-type typevar)
+	 (cons 'nesl::pair calls))
+	((and (listp return-type) (eql (car return-type) 'nesl::vector)
+	      (eql (second return-type) typevar))
+	 (let ((g1 (gensym)) (g2 (gensym)))
+	   `(with (((nesl::vector (nesl::pair seg ,g1)) ,(first calls))
+		   ((nesl::vector (nesl::pair seg ,g2)) ,(second calls)))
+	      (nesl::vector (nesl::pair seg (nesl::pair ,g1 ,g2))))))
+	(t (nesl-error "Default poly-typecase can't return the type ~a." 
+		       return-type))))
