@@ -41,7 +41,8 @@
   bugs
   redefine
   number
-  infix)
+  infix
+  codetype)
 
 (defstruct typedef
   name
@@ -69,7 +70,7 @@
 ;;;;;;;;;;;;;;;;;;;;;
 
 (defun make-function-def (names definitions 
-				&key type documentation shortdoc 
+				&key type codetype documentation shortdoc 
 				example redefine infix)
   (let* ((old-fundef (get-fundef (car names) definitions))
 	 (funstruct (make-fundef :names names
@@ -79,6 +80,7 @@
 				 :example example
 				 :redefine redefine
 				 :infix infix
+				 :codetype codetype
 				 :number (if old-fundef 
 					     (1+ (fundef-number old-fundef))
 					   0))))
@@ -105,9 +107,12 @@
 	    (incf (definition-table-type-count definitions))))))
 
 (defun add-cached-code (fundef argtypes code definitions)
-  (push (cons (get-typenum argtypes definitions) code)
-	(fundef-fun-cache fundef))
-  code)
+  (let ((rcode (if (and (listp code) (listp (car code)))
+		   (make-string-code fundef argtypes code definitions)
+		 code)))
+    (push (cons (get-typenum argtypes definitions) rcode)
+	  (fundef-fun-cache fundef))
+    rcode))
 
 (defun get-cached-code (fundef type definitions)
   (let ((typenum (get-typenum type definitions)))
