@@ -17,7 +17,7 @@
     '(defop <- with over if tup defrec defrecs
        bool int char v.bool v.int .bool .int v.
        write-op nesl
-       t nil f
+       t nil f copy pop
        vcode-equal)))
 
 (eval-when (compile load eval) 
