@@ -208,28 +208,28 @@ Commands for running VCODE on remote machines:~%  ~
 (defun load-nesl (filename &key (verbose t) (print nil))
   (when (not (stringp filename))
     (nesl-error "The second argument to LOAD must be a character string."))
-  (multiple-value-bind (type name) (get-filename-and-type filename)
-    (let ((filestring (namestring (merge-pathnames name))))
-      (cond ((not type)
-	     (nesl-error "File ~a does not exist." filestring))
-	    ((eql type :lisp)
-	     (load name :verbose verbose :print print))
-	    ((eql type :nesl)
-	     (with-open-file (loadstr name :direction :input)
-	       (when verbose 
-		 (format t "~%; Nesl loading ~a." filestring))
-	       (let ((*cnesl-syntax* nil))
-		 (declare (special *cnesl-syntax*))
-		 (nesl-loop loadstr :interactive nil :print print))))
-	    ((eql type :cnesl)
-	     (when (not (fboundp 'cgolread))
-	       (nesl-error "CNESL is not loaded, use (load-cnesl-syntax)."))
-	     (with-open-file (loadstr name :direction :input)
-	       (when verbose 
-		 (format t "~%; CNesl loading ~a." filestring))
-	       (let ((*cnesl-syntax* t))
-		 (declare (special *cnesl-syntax*))
-		 (nesl-loop loadstr :interactive nil :print print))))))))
+  (multiple-value-bind (type name) 
+    (get-filename-and-type (namestring (merge-pathnames filename)))
+    (cond ((not type)
+	   (nesl-error "File ~a does not exist." name))
+	  ((eql type :lisp)
+	   (load name :verbose verbose :print print))
+	  ((eql type :nesl)
+	   (with-open-file (loadstr name :direction :input)
+	     (when verbose 
+	       (format t "~%; Nesl loading ~a." name))
+	     (let ((*cnesl-syntax* nil))
+	       (declare (special *cnesl-syntax*))
+	       (nesl-loop loadstr :interactive nil :print print))))
+	  ((eql type :cnesl)
+	   (when (not (fboundp 'cgolread))
+	     (nesl-error "CNESL is not loaded, use (load-cnesl-syntax)."))
+	   (with-open-file (loadstr name :direction :input)
+	     (when verbose 
+	       (format t "~%; CNesl loading ~a." name))
+	     (let ((*cnesl-syntax* t))
+	       (declare (special *cnesl-syntax*))
+	       (nesl-loop loadstr :interactive nil :print print)))))))
 
 (defun trans-nesl-to-cnesl (filename)
   (let ((infile (concatenate 'string filename ".nesl"))
