@@ -161,7 +161,7 @@
   (concatenate 'string *vcode-temp-directory* name (user-name)))
 
 (defun temp-file-num (name number)
-  (temp-file (format nil "~a_~a" name (user-name) number)))
+  (format nil "~a~a~a_~a" *vcode-temp-directory* name (user-name) number))
 
 (defparameter cray-mem-add-factor 4)
 
