@@ -42,17 +42,12 @@
 ;;; DEFINITION OF THE MACRO CHARACTERS #v AND #u
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(eval-when (compile eval load)
-  (defparameter vcode-vector-letter #\u
+(defparameter vcode-vector-letter #\u
     "The letter used for low-level vcode-vectors.")
 
-  (defparameter vcode-sequence-letter #\v
+(defparameter vcode-sequence-letter #\v
     "The letter used for higher-level nestable vcode-vectors.")
 
-  (set-dispatch-macro-character #\# vcode-vector-letter 'read-vcode-vector)
-
-  (set-dispatch-macro-character #\# vcode-sequence-letter 'read-vcode-sequence))
-
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; DEFINITION OF STRUCTURES
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -170,6 +165,10 @@
 	  (flatten-nesl-type (sequence-reader stream)))
       (sequence-reader stream))))
 
+(set-dispatch-macro-character #\# vcode-vector-letter #'read-vcode-vector)
+
+(set-dispatch-macro-character #\# vcode-sequence-letter #'read-vcode-sequence)
+
 (defun sequence-reader (stream)
   (cond ((char= (peek-char nil stream) #\.)
 	 (read-char stream t nil t)
