@@ -289,7 +289,7 @@ int index;
 		    fprintf(stderr, " %s", *((int *)cpe->vector + i) ? "T" : "F");
 		    break;
 		case Float:
-		    fprintf(stderr, " %f", *(((double *)cpe->vector) + i));
+		    fprintf(stderr, " %.14e", *(((double *)cpe->vector) + i));
 		    break;
 		case String:
 		    fprintf(stderr, "%c", *(char*)(cpe->vector+i));
