@@ -103,10 +103,6 @@ char **argv;
     char *filename = NULL;
     FILE *file;
 
-#ifdef CM5
-    CVL_init();			/* set up file modes before anything else */
-    printf("done with CVL init\n");
-#endif
     while ((c = getopt(argc, argv, "cd:gm:r:t")) != EOF) {
 	switch (c) {
 	    case 'c':
@@ -190,6 +186,9 @@ static void initialize()
     CVL_init(0);			/* safety mode off -- fast'n'deadly */
 #endif DEBUG
 #endif CM2
+#ifdef CM5
+    CVL_init();
+#endif
     assert(ok_vcode_table());		/* This doesn't do anything yet */
     stack_init();
     vstack_init(vstack_size_init);
