@@ -47,8 +47,6 @@
 #include "constant.h"
 #include "stack.h"
 
-int *read_array;		/* input buffer for do_read() */
-int input_size_max = 1000;	/* size of buffer */
 
 /* The stack is an array of stack_entry's (which are pointers
  * to the vb of the associated vector).  stack_size is an index
@@ -73,7 +71,7 @@ static stack_entry_t new_se_real ()
 {
     stack_max *= 2;			/* double size */
     stack = (vb_t **) 
-	    realloc(stack, stack_max * sizeof(stack_entry_t));
+	    realloc(stack, stack_max * sizeof(vb_t*));
     if (stack == NULL) {
 	fprintf(stderr, "vinterp: error allocating internal structures (stack_entry).\n");
 	vinterp_exit (1);
@@ -90,7 +88,7 @@ static stack_entry_t new_se_real ()
 static void se_alloc_init()
 {
     stack = (vb_t **)
-	    malloc (STACK_MAX_INITIAL * sizeof (stack_entry_t));
+	    malloc (STACK_MAX_INITIAL * sizeof (vb_t*));
     if (stack == NULL) {
         fprintf(stderr, "vinterp: error allocating stack (se_alloc_init).\n");
         vinterp_exit (1);
@@ -104,12 +102,6 @@ void stack_init()
 {
     se_alloc_init();
 
-    /* allocate input buffer */
-    read_array = (int *)malloc (input_size_max*sizeof(int));
-    if (read_array == NULL) {
-	fprintf(stderr, "vinterp: can't malloc read buffer.\n\tRerun with smaller -r value.\n");
-	vinterp_exit(1);
-    }
 }
     
 /* Pop an item from the stack: first, pop out vstack entry, then 
@@ -124,6 +116,11 @@ stack_entry_t se;
 
     vstack_pop(stack[se]);
     /* copy each element into the previous one */
+/*
+#ifdef cray
+#pragma ivdep
+#endif
+*/
     for (to_se = se; to_se < stack_index; to_se ++)
 	stack[to_se] = stack[to_se + 1];
     
@@ -224,13 +221,29 @@ int start,	/* 0 is top of stack */
     }
 	
     /* copy each se until done */
-    for (from = stack_index - start - count + 1, last = stack_index - start;
-	 from <= last;
-	 from ++) {
-	se = new_se();
-	stack[se] = stack[from];
-	vstack_copy(se_vb(se));
+    from = stack_index - start - count + 1;
+    last = stack_index - start;
+
+#ifdef SPEEDHACKS
+    /* if no storage problems, can do fast loop */
+    if (count < stack_max - stack_index) {
+	/* NB: This loop doesn't vectorize because of loop carried
+	 * dependency on the count++
+	 */
+	for (; from <= last; from ++) {
+	    stack[++stack_index] = stack[from];
+	    stack[stack_index]->count++;
+	}
+    } else {
+#endif /* SPEEDHACKS */
+	for (; from <= last; from ++) {
+	    se = new_se();
+	    stack[se] = stack[from];
+	    vstack_copy(se_vb(se));
+	}
+#ifdef SPEEDHACKS
     }
+#endif /* SPEEDHACKS */
 }
 
 /* Pop out values from middle of stack:
@@ -259,6 +272,11 @@ int count,
     }
 
     /* copy the remaining elements up */
+/*
+#ifdef cray
+#pragma ivdep
+#endif
+*/
     for (from = stack_index - start + 1; from <= stack_index; from ++) {
 	stack[from-count] = stack[from];	/* move things count back */
     }
@@ -280,6 +298,11 @@ stack_entry_t arg_array[];
 	vinterp_exit (1);
     }
 
+/*
+#ifdef cray
+#pragma ivdep
+#endif
+*/
     for (i = 0; i < numargs; i++) {
 	arg_array[numargs - i - 1] = stack_index - i;
     }
@@ -393,6 +416,8 @@ FILE *stream;
 }
 
 
+#if 0
+/* THIS IS USED ANY MORE.  HOLD IT UNTIL WE DEFINATELY DON'T NEED IT */
 /* get a value of given type from stream, and assign to buffer.
  * returns pointer to location to put next result, or NULL if
  * data wasn't found or if the end of the vector was encountered.
@@ -448,6 +473,7 @@ FILE *stream;
 	}
     return NULL;
 }
+#endif
 
 /* perform the WRITE function; write_value() and write_string() do the work.
  * This puts two values on the stack: 
@@ -486,16 +512,13 @@ void do_read(pe, stream)
 prog_entry_t *pe;
 FILE *stream;
 {
-    int count = 0;			/* total number elements read */
+    int count;			        /* total number elements read */
     int i;
     vec_p vector;			/* where to copy input */
     TYPE type = pe->type;
-    TYPE read_type;			/* type of data being read: for type
-					 * Segdes -> Int  */
     TYPE stack_type;			/* type of vector element:
 					 * String -> Int */
     int c;				/* char read in */
-    int *new_value_buffer;		/* for reading in data */
 
     char error_msg[256];
     int error_flag= 0;
@@ -503,6 +526,9 @@ FILE *stream;
     int start_vector;			/* must appear as first char */
     int end_vector;			/* must appear as last char */
 
+    int *read_array;			/* input buffer */
+    int input_size_max = 1000;		/* size of buffer */
+
     /* print out a nice message if working interactively */
     if (isatty(fileno(stream))) {
 	char *type_string = NULL, *form_string = NULL;
@@ -535,21 +561,22 @@ FILE *stream;
         fprintf(stderr, "vinterp: type %s vector, in form %s: ", type_string, form_string);
     }
 
-    /* figure out vector delimiters, and types to be read and stored */
+    /* skip whitespace */
+    while ((c = getc(stream)) != EOF && isspace(c))
+	;
+
     switch (type) {
 	case Int: case Float: case Bool:
 	    start_vector = '('; end_vector = ')';
-	    stack_type = read_type = type;
+	    stack_type = type;
 	    break;
 	case String: 
 	    end_vector = start_vector = '"';
-	    read_type = String;
 	    stack_type = Int;
 	    break;
 	case Segdes:
 	    start_vector = '['; end_vector = ']';
-	    read_type = Int;
-	    stack_type = Segdes;
+	    stack_type = Int;
 	    break;
 	default:
 	    fprintf(stderr, "vinterp: internal error: improper type %d in do_read().\n", type);
@@ -557,10 +584,6 @@ FILE *stream;
 	    break;
     }
 
-    /* skip whitespace */
-    while ((c = getc(stream)) != EOF && isspace(c))
-	;
-
     /* get start of vector */
     if (c != start_vector) {
 	error_flag = 1;
@@ -568,45 +591,129 @@ FILE *stream;
 	goto ERROR;
     }
 
-    new_value_buffer = read_array;
-    count = 0;
-
-    /* read values until we're done */
-    while (1) {
-	/* check to see if room in buffer, reallocate if not */
-	if (new_value_buffer >= read_array + input_size_max) {
-	    input_size_max *= 2;
-	    read_array = (int *)realloc((char*)read_array, 
-		     	                input_size_max * sizeof(int));
-	    if (read_array == NULL) {
-	        fprintf(stderr, "vinterp: could not reallocate read buffer.\n");
-	        vinterp_exit(1);
-	    } else {
-		switch (read_type) {
-		    case Int: case Bool: case String:
-			new_value_buffer = read_array + count;
-			break;
-		    case Float:
-			new_value_buffer = (int*)(count + (double*)read_array);
-			break;
-		    default:
-			fprintf(stderr, "vinterp: internal error: bad type in do_read()\n");
-			vinterp_exit(1);
-		}
-	    }
-	}
-
-	/* do the input */
-	new_value_buffer = get_value(new_value_buffer, read_type, stream);
-	if (new_value_buffer == NULL)
-	    break;
-	count ++;
+    /* allocate input buffer */
+    read_array = (int *)malloc (input_size_max*sizeof(int));
+    if (read_array == NULL) {
+	fprintf(stderr, "vinterp: can't malloc read buffer.\n");
+	vinterp_exit(1);
     }
 
-    /* skip whitespace */
-    while ((c = getc(stream)) != EOF && isspace(c))
-	;
+    count = 0;
+    /* read routines for each type.  each of these leaves the char that
+     * stopped the read (EOF, vector delimiter, etc) in c,
+     * and sets count to the number of items read.
+     */
+    switch (type) {
+	case Int: case Segdes:
+	    /* write our own integer parser to avoid fscanf overhead */
+	    c = getc(stream);
+	    while (1) {			/* read in new integer */
+		int val = 0;		/* int being read */
+	        int ispositive = 1;	/* positive or negative */
 
+		while (isspace(c)) { c = getc(stream);}
+		if (isdigit(c)) goto DIGIT;
+		else if (c == '-') ispositive = 0;
+		else if (c != '+') goto DONE;
+DIGIT:
+		do {				/* read digits */
+		    val = 10*val + (c - '0');
+		    c = getc(stream);
+		} while (isdigit(c));
+
+		if (count < input_size_max) {	/* put into read buffer */
+		    read_array[count] = ispositive ? val : -val;
+		} else {				/* expand buffer */
+		    input_size_max *= 2;
+		    read_array = (int *)realloc((char*)read_array, 
+						input_size_max * sizeof(int));
+		    if (read_array == NULL) {
+			fprintf(stderr, "vinterp: could not reallocate read buffer.\n");
+			vinterp_exit(1);
+		    } else {
+		        read_array[count] = ispositive ? val : -val;
+		    }
+		}
+		count ++;
+	    }
+	    break;   	/* int case */
+	
+	case Bool:
+	    while (1) {				/* read in bools */
+	    int val;
+		c = getc(stream);
+		switch(c) {
+		    case ' ': case '\t': case '\n': continue;
+		    case 'T': val = 1; break;
+		    case 'F': val = 0; break;
+		    default: goto DONE;
+		}
+		if (count < input_size_max) {	/* put into read buffer */
+		    read_array[count] = val;
+		} else {				/* expand buffer */
+		    input_size_max *= 2;
+		    read_array = (int *)realloc((char*)read_array, 
+						input_size_max * sizeof(int));
+		    if (read_array == NULL) {
+			fprintf(stderr, "vinterp: could not reallocate read buffer.\n");
+			vinterp_exit(1);
+		    }
+		    read_array[count] = val;
+		}
+		count ++;
+	    }
+	    break;
+
+	case String:
+	    /* For now, just get a char.  Eventually, we would like '\' 
+	     * to do something useful.
+	     */
+	    while (1) {
+	        c = getc(stream);
+		if ((c == (int) '\"') || (c != EOF)) goto DONE;
+
+		if (count < input_size_max) {	/* put into read buffer */
+		    read_array[count] = c;
+		} else {			/* expand buffer */
+		    input_size_max *= 2;
+		    read_array = (int *)realloc((char*)read_array, 
+						input_size_max * sizeof(int));
+		    if (read_array == NULL) {
+			fprintf(stderr, "vinterp: could not reallocate read buffer.\n");
+			vinterp_exit(1);
+		    }
+		    read_array[count] = c;
+		}
+		count ++;
+	    }
+	    break;
+	
+        case Float:
+	    while (1) {
+		double val;
+		if (fscanf(stream, " %lf", &val) != 1) {
+		    c = getc(stream);
+		    goto DONE;
+		} else {
+		    if (count*sizeof(double) < input_size_max) { 
+			((double *)read_array)[count] = val;
+		    } else {			/* expand bugger */
+			input_size_max *= 2;
+			read_array = (int *)realloc((char*)read_array, 
+						    input_size_max * sizeof(int));
+			if (read_array == NULL) {
+			    fprintf(stderr, "vinterp: could not reallocate read buffer.\n");
+			    vinterp_exit(1);
+			}
+			((double *)read_array)[count] = val;
+		    }
+		    count ++;
+		}
+	    }
+	    break;
+    } /* end switch */
+
+DONE:
     /* get end of vector */
     if (c != end_vector) {
 	error_flag = 1;
@@ -636,8 +743,7 @@ ERROR:
     vector = se_vb(TOS)->vector;
     for (i=0; i < count; i++) {		/* copy values in, one by one ...  */
 	switch (stack_type) {
-	    case Int:
-	    case Segdes: 	/* create the lengths vector for a segdes */
+	    case Int: 			/* this covers Segdes case as well */
 		assert_mem_size(rep_vuz_scratch(count));
 		rep_vuz(vector, i, read_array[i], count, SCRATCH);
 		break;
@@ -654,6 +760,10 @@ ERROR:
 		vinterp_exit (1);
 	}
     }
+
+    /* free read_array */
+    free(read_array);
+
     if (type == Segdes) {
 	/* do a MAKE_SEGDES from the length vector */
 	int vector_len;
@@ -1090,6 +1200,7 @@ prog_entry_t *pe;
 
     int child_in_fd[2], child_out_fd[2], child_err_fd[2];  /* for pipe */
 
+    char **argv;				/* put command here (malloc) */
     int isParent;				/* return of fork() */
 
     /* get the various fd from the stack */
@@ -1126,6 +1237,30 @@ prog_entry_t *pe;
     stack_pop(TOS);			/* remove command name from stack */
 
     /* Now we can do the spawning */
+
+    { 
+        /* set up argument list for call from command */
+
+	int argc;	/* arg count for subprocess */
+	int i;
+	int inspace;	/* for command parser */
+
+	/* must break the command string up into pieces */
+	argv = (char **) malloc (sizeof (char *) * command_len);
+	argc = 0;
+	inspace = 1;	/* start on space to catch beginning */
+	for (i = 0; i < command_len; i++) {
+	    if (isspace(command[i])) {
+		command[i] = NULL;
+		inspace = 1;
+	    } else if (inspace) {
+		/* found new word: went from space to non-space */
+		argv[argc++] = command + i;
+		inspace = 0;
+	    }
+	}
+	argv[argc] = NULL;
+    }
     
     /* For each NULL_STREAM argument, we must set up a pipe
      * so that the two processes can communicate.
@@ -1135,7 +1270,7 @@ prog_entry_t *pe;
     if (err_fd == NULL_STREAM_FD) pipe(child_err_fd);
     
     /* Second, fork off a subprocess */
-    isParent = fork();
+    isParent = vfork();
 
     if (isParent) { 	/* parent process will be the interpreter */
 	/* if NULL_STREAM_FD was used, get the corect end of the pipe,
@@ -1168,27 +1303,6 @@ prog_entry_t *pe;
 	    error_flag = 1;
 	}
     } else {		/* child will exec the command */
-	int argc;	/* arg count for subprocess */
-	char **argv;
-	int i;
-	int inspace;	/* for command parser */
-
-	/* must break the command string up into pieces */
-	argv = (char **) malloc (sizeof (char *) * command_len);
-	argc = 0;
-	inspace = 1;	/* start on space to catch beginning */
-	for (i = 0; i < command_len; i++) {
-	    if (isspace(command[i])) {
-		command[i] = NULL;
-		inspace = 1;
-	    } else if (inspace) {
-		/* found new word: went from space to non-space */
-		argv[argc++] = command + i;
-		inspace = 0;
-	    }
-	}
-	argv[argc] = NULL;
-
 	/* Set up IO strams for child.  If NULL_STREAM was specified, 
 	 * make correct end of pipe into appropriate stream, and close other
 	 * end.  Otherwise, use supplied stream. 
@@ -1201,13 +1315,13 @@ prog_entry_t *pe;
 
 	if (out_fd == NULL_STREAM_FD) {
 	    dup2(child_out_fd[1], fileno(stdout));
-	    close(child_out_fd[2]);
+	    close(child_out_fd[0]);
 	} else 
 	    dup2(fileno(file_list[out_fd]), fileno(stdout));
 
 	if (err_fd == NULL_STREAM_FD) {
 	    dup2(child_err_fd[1], fileno(stderr));
-	    close(child_err_fd[2]);
+	    close(child_err_fd[0]);
 	} else 
 	    dup2(fileno(file_list[err_fd]), fileno(stderr));
 
@@ -1248,5 +1362,154 @@ ERROR:
     stack_push(1, Bool, -1);		/* negation of error_flag */
     assert_mem_size(rep_vub_scratch(1));
     rep_vuz(se_vb(TOS)->vector, 0, (cvl_bool) !error_flag, 1, SCRATCH);
+
+    free(command);
+    free(argv);
 }
 
+
+/* Read from a character stream. 
+ * Takes three args from stack:
+ *   stream to read from
+ *   int giving maximum number of chars to read (negative means don't worry)
+ *   char vector giving list of delimiters (empty means don't worry)
+ * Returns three values on the stack:
+ *   bool - T if read is successful
+ *   string - error message if failure in read, null otherwise
+ *   vector - result of the read
+ */   
+
+void do_fread_char(pe)
+prog_entry_t *pe;
+{
+    vb_t *fd_vb = se_vb(TOS);		/* get fd from stack */
+    int fd;
+    FILE *file;				/* from file_list */
+    int max_char;			/* max number of chars to read */
+    vb_t *max_char_vb;
+    vb_t *delim_vb;			/* delimiters for intput */
+    int delim_len;			/* number of delimiters */
+    int delim_mask[256];
+    int i;
+    int c;
+    int *read_array;			/* input buffer */
+    int input_size_max = 1000;		/* size of buffer */
+
+    /* check for integer scalar value */
+    if (check_args && (fd_vb->type != Int || fd_vb->len != 1)) {
+	fprintf(stderr, "vinterp: line %d: FREAD_CHAR: top of stack not integer scalar. \n", pe->lineno);
+	vinterp_exit(1);
+    }
+
+    /* get fd off of stack */
+    assert_mem_size(ext_vuz_scratch(1));
+    fd = (int) ext_vuz(fd_vb->vector, 0, 1, SCRATCH);
+
+    stack_pop(TOS);			/* remove fd from stack */
+
+    file = file_list[fd];		/* get actual file handler */
+
+    if (file == NULL) {			/* check for validity */
+	/* invalid: write results to stack */
+	char *fread_error = "illegal VCODE file descriptor in FREAD";
+	int i;
+	int len = strlen(fread_error);
+
+	stack_push(0, pe->type, -1);	/* null result */
+
+	stack_push(len, Int, -1);	/* error message */
+	assert_mem_size(rep_vuz_scratch(1));
+	for (i = 0; i < len; i++) {		/* put string on stack */
+	    rep_vuz(se_vb(TOS)->vector, i, (int) fread_error[i], len, SCRATCH);
+	}
+
+	stack_push(1, Bool, -1);	/* error flag */
+	assert_mem_size(rep_vub_scratch(1));
+	rep_vub(se_vb(TOS)->vector, 0, (cvl_bool) 0, 1, SCRATCH);
+	return;
+    }
+
+    /* set up max char */
+    max_char_vb = se_vb(TOS);		
+    if (check_args && (max_char_vb->type != Int || max_char_vb->len != 1)) {
+	fprintf(stderr, "vinterp: line %d: FREAD_CHAR: second element on stack not scalar int.\n", pe->lineno);
+	vinterp_exit(1);
+    }
+    assert_mem_size(ext_vuz_scratch(1));
+    max_char = (int) ext_vuz(max_char_vb->vector, 0, 1, SCRATCH);
+    if (max_char < 0) max_char = 1 << 30;	/* no limit, essentially */
+    stack_pop(TOS);
+
+    /* get delimiters */
+    delim_vb = se_vb(TOS);
+    if (check_args && (delim_vb->type != Int)) {
+	fprintf(stderr, "vinterp: line %d: FREAD_CHAR: third element on stack not string.\n", pe->lineno);
+	vinterp_exit(1);
+    }
+    assert_mem_size(ext_vuz_scratch(1));
+
+    /* zero out delim mask */
+    for (i = 0; i < 256; i++) delim_mask[i] = 0;
+
+    /* create mask */
+    if (check_args) {		/* check each element */
+	for (i = 0; i < delim_vb->len; i++) {
+	    unsigned int delim = ((int *)delim_vb->vector)[i];
+	    if (delim < 256) 
+	        delim_mask[delim] = 1;
+	    else {
+		fprintf(stderr, "vinterp: line %d: delimiter element %d is not a legal character. \n", pe->lineno, i);
+		vinterp_exit(1);
+	    }
+	}
+    } else {
+	for (i = 0; i < delim_vb->len; i++) {
+	    assert(((int *)delim_vb->vector)[i] < 256);
+	    delim_mask[((int *)delim_vb->vector)[i]] = 1;
+	}
+    }
+    stack_pop(TOS);
+
+    /* allocate input buffer */
+    read_array = (int *)malloc (input_size_max*sizeof(int));
+    if (read_array == NULL) {
+	fprintf(stderr, "vinterp: can't malloc read buffer.\n");
+	vinterp_exit(1);
+    }
+
+    /* Read chars from file until EOF, delimiter or max_char is found */
+    i = 0;
+    while ((i < max_char) && 			/* too many chars */
+	   ((c = getc(file)) != EOF) && 	/* EOF */
+	   delim_mask[c] == 0) {		/* no delimiter */
+
+	/* check to see if room in buffer, reallocate if not */
+	if (i == input_size_max) {
+	    input_size_max *= 2;
+	    read_array = (int *)realloc((char*)read_array, 
+		     	                input_size_max * sizeof(int));
+	    if (read_array == NULL) {
+	        fprintf(stderr, "vinterp: could not reallocate read buffer.\n");
+	        vinterp_exit(1);
+	    }
+	}
+
+        read_array[i] = c;
+	i++;
+    }
+
+    /* put string on stack */
+    stack_push(i, Int, -1);		/* allocate stack space */
+    for (c=0; c < i; c++) {		/* copy values in, one by one ...  */
+	assert_mem_size(rep_vuz_scratch(i));
+	rep_vuz(se_vb(TOS)->vector, c, read_array[c], i, SCRATCH);
+    }
+
+    free(read_array);
+
+    stack_push(0, Int, -1);     /* put null error string on stack */
+
+    stack_push(1, Bool, -1);	/* bool return code on stack */
+    assert_mem_size(rep_vuz_scratch(i));
+    rep_vuz(se_vb(TOS)->vector, 0, (cvl_bool)1, 1, SCRATCH);
+}
